<?php

namespace Bydedi\Cache;

/**
 * Class PhpCache
 *
 * @package Cache
 */
abstract class PhpCache implements CacheInterface
{

    /**
     * @var string
     */
    protected $phpcache_dir;
    /**
     * @var bool
     */
    protected $enable;

    /**
     * construct
     */
    public function __construct()
    {
        $this->enable = getenv('APP_CACHE');
    }

    /**
     * get data in cache
     *
     * @param string $filename  the name of the file
     * @param int    $timeCache time to cache data
     *
     * @return bool|mixed
     */
    public function get($filename, $timeCache = 900)
    {
        if ($this->enable == false) {
            return false;
        }

        $this->createDirIfNotExist();

        $chemin = $this->phpcache_dir . DS . $filename . ".php";

        if (file_exists($chemin)) {
            $timeFile = filemtime($chemin);
            $newTime = time() - $timeCache;
            if ($newTime < $timeFile) {
                return unserialize(file_get_contents($chemin));
            }
        }

        return false;
    }

    /**
     * Create a file with data to cached
     *
     * @param string $filename the name of the file
     * @param mixed  $var      data to cached
     *
     * @return bool
     */
    public function create($filename, $var)
    {
        if ($this->enable == false) {
            return false;
        }

        $this->createDirIfNotExist();

        $chemin = $this->phpcache_dir . DS . $filename . ".php";

        file_put_contents($chemin, serialize($var));
    }

    /**
     * clean file cache
     *
     * @return void
     */
    public function clear()
    {
        $mask = $this->phpcache_dir . DS . "*.php";
        array_map("unlink", glob($mask));
    }

    /**
     * Create the directory where the data in cache where store
     *
     * @return void
     */
    protected function createDirIfNotExist()
    {
        if (!file_exists($this->phpcache_dir)) {
            mkdir($this->phpcache_dir, 0775, true);
        }
    }
}
