<?php

namespace Bydedi\Container;

use Symfony\Component\Config\FileLocator;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\DependencyInjection\Definition;
use Symfony\Component\DependencyInjection\Loader\YamlFileLoader;
use Symfony\Component\EventDispatcher\DependencyInjection\RegisterListenersPass;

/**
 * Class ServiceContainer
 *
 * @package Bydedi\Container
 *
 */
class ServiceContainer extends ContainerBuilder
{
    /**
     * @var ServiceProvider[]
     */
    protected $serviceProviders;

    public function __construct()
    {
        parent::__construct();

        $this->addCompilerPass(
            new RegisterListenersPass('event.dispatcher', 'app.event_listener', 'app.event_subscriber')
        );

        $this->setDefinition('service_container', new Definition())
            ->setSynthetic(true);

        $loader = new YamlFileLoader($this, new FileLocator(__DIR__));
        $loader->load('config/services.yml');
    }

    /**
     * @param array() $providers
     */
    public function registerProviders($providers)
    {
        $this->serviceProviders = array();

        foreach ($providers as $provider) {
            $providerObject = new $provider($this);
            $providerObject->register();
            $this->serviceProviders[] = $providerObject;
        }
    }
}
