<?php

// TODO : revoir la class pour utiliser l'entity manager

namespace Bydedi\Utils\I18n;

use Doctrine\ORM\EntityManagerInterface;

/**
 * Class Text
 * @package Bydedi\Utils\I18n
 */
class Text
{
    /**
     * @var array
     */
    protected $texts;

    /**
     * @var EntityManagerInterface
     */
    protected $entityManager;

    /**
     * @param EntityManagerInterface $entityManager
     */
    public function __construct(EntityManagerInterface $entityManager)
    {
        $this->entityManager = $entityManager;

        // load all texts in cache on construct
        // $this->loadAllTexts();
    }

    /**
     * @param array $formvars
     * @param string $champs
     * @return array|string
     *
     * recuperation des infos de texte
     */
    public function get($formvars = array(), $champs = "")
    {

        $query = new Query();
        $query->select()->from('text')
            ->leftJoin(
                'text_i18n',
                'AND',
                array('t.id', 'ti.text_id'),
                array('ti.lang', '"' . getenv('APP_LOCALE') . '"')
            );
        $query->extras($formvars);

        if (empty($champs)) {
            return $this->connection->getAll($query->build());
        } else {
            $result = $this->connection->getOne($query->build());
            return (!empty($result[$champs])) ? $result[$champs] : '';
        }
    }

    /**
     * find a text by code
     *
     * @param $code
     * @return mixed
     */
    public function getOne($code)
    {
        if (array_key_exists($code, $this->texts) == false) {
            $this->texts[$code] = $this->get(array('code' => $code), 'value');
        }

        return $this->texts[$code];

    }

    public function trans($text, $key = null)
    {
        if (is_null($key)) {
            $key = $this->generateCode($text);
        }

        $value = $this->getOneOrCreate($key, $text);

        return $value;
    }

    /**
     * @param $key
     * @param $value
     * @return mixed
     */
    public function getOneOrCreate($key, $value)
    {
        $foundText = $this->getOne($key);

        // we've got something, return it
        if (!empty($foundText)) {
            return $foundText;
        }

        // add text in db
        $this->add(['code' => $key, 'value' => $value]);

        // update our array
        $this->texts[$key] = $value;

        return $value;
    }

    /**
     * Load all texts in the cache object
     */
    public function loadAllTexts()
    {
        $this->texts = [];

        $results = $this->get();

        foreach ($results as $value) {
            $this->texts[$value['code']] = $value['value'];
        }
    }

    /**
     * @param array $formvars
     * @return mixed
     * @throws \Exception
     */
    public function add($formvars = array())
    {

        $q = new Query();
        $q->select()
            ->from('text')
            ->where('code', $formvars['code']);

        $ed = $this->connection->getOne($q->build());

        if (empty($ed)) {
            $edId = $this->connection->save('text', ['code' => $formvars['code']]);
        } else {
            $edId = $ed['id'];
        }

        if (!empty($edId)) {
            $this->connection->save('text_i18n', array(
                'text_id' => $edId,
                'value' => $formvars['value'],
                'lang' => getenv('APP_LOCALE'),
            ));
        }

        return $edId;
    }

    /**
     * @param $value    texte dans la langue par defaut
     * @return string   la clé pour de la gestion des textes
     */
    public function generateCode($value)
    {
        $value = strip_tags($value);
        $key = trim(mb_strtolower($value));

        $table = array(
            'Š' => 'S', 'š' => 's', 'Đ' => 'D', 'đ' => 'd', 'Ž' => 'Z', 'ž' => 'z', 'Č' => 'C', 'č' => 'c', 'Ć' => 'C',
            'ć' => 'c',
            'À' => 'A', 'Á' => 'A', 'Â' => 'A', 'Ã' => 'A', 'Ä' => 'A', 'Å' => 'A', 'Æ' => 'A', 'Ç' => 'C', 'È' => 'E',
            'É' => 'E',
            'Ê' => 'E', 'Ë' => 'E', 'Ì' => 'I', 'Í' => 'I', 'Î' => 'I', 'Ï' => 'I', 'Ñ' => 'N', 'Ò' => 'O', 'Ó' => 'O',
            'Ô' => 'O',
            'Õ' => 'O', 'Ö' => 'O', 'Ø' => 'O', 'Ù' => 'U', 'Ú' => 'U', 'Û' => 'U', 'Ü' => 'U', 'Ý' => 'Y', 'Þ' => 'B',
            'ß' => 'Ss',
            'à' => 'a', 'á' => 'a', 'â' => 'a', 'ã' => 'a', 'ä' => 'a', 'å' => 'a', 'æ' => 'a', 'ç' => 'c', 'è' => 'e',
            'é' => 'e',
            'ê' => 'e', 'ë' => 'e', 'ì' => 'i', 'í' => 'i', 'î' => 'i', 'ï' => 'i', 'ð' => 'o', 'ñ' => 'n', 'ò' => 'o',
            'ó' => 'o',
            'ô' => 'o', 'õ' => 'o', 'ö' => 'o', 'ø' => 'o', 'ù' => 'u', 'ú' => 'u', 'û' => 'u', 'ý' => 'y', 'þ' => 'b',
            'ÿ' => 'y', 'Ŕ' => 'R', 'ŕ' => 'r', ' - ' => '-', ' ' => '_', '&' => 'et', 'œ' => 'oe', "'" => '', '"' => ''
        );

        $key = strtr($key, $table);

        $key = preg_replace('/[^a-z0-9-_]/', '', $key);

        return $key;
    }
}
